#!/usr/bin/ruby -s

require "digest/md5"
require "digest/sha1"
require "digest/sha2"
require "digest/rmd160"

require "util.rb"
require "datatype.rb"

if $h
  $stderr.print "Usage: " + $0 + " [-md5|-sha1|-sha256|-sha384|-sha512|-rmd160|-pgp]\n"
  exit 1
end


$width=32


def md5sign(str)
  keystr = Digest::MD5.new(str).hexdigest
  ret = str_split(keystr, $width).join(".")
  return ret
end
def sha1sign(str)
  keystr = Digest::SHA1.new(str).hexdigest
  ret = str_split(keystr, $width).join(".")
  return ret
end
def sha256sign(str)
  keystr = Digest::SHA256.new(str).hexdigest
  ret = str_split(keystr, $width).join(".")
  return ret
end
def sha384sign(str)
  keystr = Digest::SHA384.new(str).hexdigest
  ret = str_split(keystr, $width).join(".")
  return ret
end
def sha512sign(str)
  keystr = Digest::SHA512.new(str).hexdigest
  ret = str_split(keystr, $width).join(".")
  return ret
end
def rmd160sign(str)
  keystr = Digest::RMD160.new(str).hexdigest
  ret = str_split(keystr, $width).join(".")
  return ret
end

def pgpsign(str)
  ret = ""

  open("|./pgp-wrapper.sh pass '" + str.dump[1..str.dump.length-2] + "'", "r") {
    |fd|
    l = fd.read
    keystr = l.unpack("H*")[0]
    ret = str_split(keystr, $width).join(".")
  }

  return ret
end

def gen_sign(s)
  ret = {}

  ret["md5"]    = md5sign(s)    if $md5
  ret["sha1"]   = sha1sign(s)   if $sha1
  ret["sha256"] = sha256sign(s) if $sha256
  ret["rmd160"] = rmd160sign(s) if $rmd160
  ret["pgp"]    = pgpsign(s)    if $pgp

  return ret
end


while line = gets
  if line =~ /sec\-auth\-/
    next
  end
  if line =~ /^#/
    next
  end


  if /^=[a-z]/ =~ line
    (name, addr, ttl) = line.chop[1..line.length].split(":")

    ttl = "" if ttl == nil
    ipstr = str_to_ipstr(addr)
    dnsname = str_to_dnsstr(name)
    dummy_ttl = "\0\0\0\0"

    s = dnsname + DnsType::A + DnsClass::INET + dummy_ttl + [ipstr.length].pack("n") + ipstr

    print line
    gen_sign(s).each {
      |i|
      print "=sec-auth-#{i[0]}--.#{i[1]}.--.#{name}:#{addr}:#{ttl}\n"
    }
    next
  end

  if /^\./ =~ line
    (name, addr, prefix, ttl) = line.chop[1..line.length].split(":")

    #next if /in\-addr\.arpa$/ =~ name

    ipstr = str_to_ipstr(addr)
    ns = "ns." + name

    ttl = "" if ttl == nil

    if prefix[prefix.length - 1] == ?.
      ns = prefix
    elsif prefix != ""
      ns = prefix + "." + ns
    end

    dnsname = str_to_dnsstr(ns)
    dummy_ttl = "\0\0\0\0"

    s = dnsname + DnsType::A + DnsClass::INET + dummy_ttl + [ipstr.length].pack("n") + ipstr

    gen_sign(s).each {
      |i|
      print(".#{name}:#{addr}:sec-auth-#{i[0]}--.#{i[1]}.--.#{ns}.:#{ttl}\n")
    }
    next
  end

  if /^@/ =~ line
    (name, addr, prefix, pri, ttl) = line.chop[1..line.length].split(":")

    next if /in\-addr\.arpa$/ =~ name

    ipstr = str_to_ipstr(addr)
    ns = "mx." + name

    ttl = "" if ttl == nil
    pri = "" if pri == nil

    if prefix[prefix.length - 1] == ?.
      ns = prefix
    elsif prefix != ""
      ns = prefix + "." + ns
    end

    dnsname = str_to_dnsstr(ns)
    dummy_ttl = "\0\0\0\0"

    s = dnsname + DnsType::A + DnsClass::INET + dummy_ttl + [ipstr.length].pack("n") + ipstr

    gen_sign(s).each {
      |i|
      print("@#{name}:#{addr}:sec-auth-#{i[0]}--.#{i[1]}.--.#{ns}.:#{pri}:#{ttl}\n")
    }
    next
  end


  print line

end
